"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ModuleResolver = void 0;
const tslib_1 = require("tslib");
const module_1 = require("module");
const path_1 = require("path");
const node_1 = require("@idlebox/node");
const fs_extra_1 = require("fs-extra");
const typescript_1 = tslib_1.__importDefault(require("typescript"));
const ProjectConfig_1 = require("./ProjectConfig");
class ResolveResult {
    self;
    id;
    options;
    getPacakgeJson;
    logger;
    resolvedModule;
    failedLookupLocations;
    isInternal;
    constructor(self, id, options, getPacakgeJson, resolved, logger) {
        this.self = self;
        this.id = id;
        this.options = options;
        this.getPacakgeJson = getPacakgeJson;
        this.logger = logger;
        this.resolvedModule = resolved.resolvedModule;
        this.failedLookupLocations = resolved.failedLookupLocations;
        // resolutionDiagnostics
        this.isInternal = false;
        if (!this.resolvedModule) {
            if (module_1.builtinModules.includes(id)) {
                this.isInternal = true;
            }
        }
    }
    get success() {
        return !!this.resolvedModule || this.isInternal;
    }
    get isNodeModules() {
        return this.resolvedModule?.isExternalLibraryImport || false;
    }
    get moduleName() {
        return this.resolvedModule?.packageId?.name;
    }
    get modulePath() {
        return this.resolvedModule?.packageId?.subModuleName;
    }
    get absoluteRealpath() {
        return this.resolvedModule?.resolvedFileName;
    }
    get absolutePathWithExtension() {
        return this.resolvedModule?.originalPath || this.resolvedModule?.resolvedFileName;
    }
    get absolutePath() {
        const abs = this.absolutePathWithExtension;
        return abs.slice(0, abs.length - this.extension.length);
    }
    get extension() {
        return this.resolvedModule?.extension || '';
    }
    get relativeToSource() {
        const r = (0, node_1.relativePath)((0, path_1.dirname)(this.self), this.absolutePath);
        if (r.startsWith('.'))
            return r;
        return './' + r;
    }
    get relativeToSourceRoot() {
        const root = this.options.rootDir || (0, path_1.dirname)(this.options.configFilePath);
        const r = (0, node_1.relativePath)(root, this.absolutePath);
        if (r.startsWith('.'))
            return r;
        return './' + r;
    }
    readPackageJson() {
        if (!this.success)
            throw new Error('not valid import target');
        if (this.isNodeModules) {
            const require = (0, module_1.createRequire)(this.self);
            const target = this.moduleName;
            try {
                return require(target + '/package.json');
            }
            catch (e) {
                if (e && (e.code === 'ERR_PACKAGE_PATH_NOT_EXPORTED' || e.code === 'PACKAGE_PATH_NOT_EXPORTED')) {
                    let packageJson = undefined;
                    let curr = require.resolve(target);
                    while (true) {
                        const file = (0, node_1.findUpUntilSync)(curr, 'package.json');
                        if (!file) {
                            packageJson = undefined;
                            throw e;
                        }
                        packageJson = (0, fs_extra_1.readJsonSync)(file);
                        if (packageJson.name === target) {
                            return packageJson;
                        }
                        curr = (0, path_1.dirname)((0, path_1.dirname)(curr));
                    }
                }
                throw e;
            }
        }
        else {
            const pkgpath = this.getPacakgeJson();
            if (pkgpath) {
                return require(pkgpath);
            }
            throw new Error(`failed find package.json (from ${this.self})`);
        }
    }
    printWhyFailed() {
        if (this.success) {
            return;
        }
        this.logger.warn('failed resolve module "%s" from (%s)\nFind Paths:\n  - %s', this.id, this.self, this.failedLookupLocations.join('\n  - '));
    }
}
// const dependenciesFields = [
// 	'dependencies',
// 	'devDependencies',
// 	'optionalDependencies',
// 	'peerDependencies',
// 	'bundledDependencies',
// ];
class ModuleResolver extends ProjectConfig_1.ProjectConfig {
    // private readonly nodeResolve: NodeRequire;
    // private readonly dependencies: Set<string>;
    // private readonly config: ProjectConfig;
    resolveCache;
    // private readonly paths: ts.CompilerOptions['paths'];
    // private readonly mapRoot: ts.CompilerOptions['mapRoot'];
    constructor(compilerOptions, logger, host) {
        super(compilerOptions, logger);
        this.resolveCache = host?.getModuleResolutionCache?.();
        if (!this.resolveCache) {
            logger.warn(new Error('missing compiler host, or missing resolve cache.').stack);
        }
        // this.mapRoot = compilerOptions.mapRoot;
        // this.paths = compilerOptions.paths;
        // this.config = new ProjectConfig(compilerOptions, logger);
        // const pkg = require(this.config.packageJson);
        // this.nodeResolve = createRequire(config.packageJson);
        // this.dependencies = new Set();
        // for (const item of dependenciesFields) {
        // 	if (pkg[item]) {
        // 		for (const key of Object.keys(pkg[item])) {
        // 			this.dependencies.add(key);
        // 		}
        // 	}
        // }
    }
    resolve(self, id) {
        return new ResolveResult(self, id, this.options, this.findPackageJson, typescript_1.default.resolveModuleName(id, self, this.options, typescript_1.default.sys, this.resolveCache, undefined, typescript_1.default.ModuleKind.CommonJS), this.logger);
    }
    getPathInfo(id) {
        const packageName = id.startsWith('@') ? id.split('/', 2).join('/') : id.split('/', 1)[0];
        const fileName = id.slice(packageName.length + 1);
        return { packageName, fileName };
    }
}
exports.ModuleResolver = ModuleResolver;
//# sourceMappingURL=ModuleResolver.js.map