"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TypescriptTransformPlugin = exports.EngineKind = void 0;
const tslib_1 = require("tslib");
const path_1 = require("path");
const common_1 = require("@idlebox/common");
const typescript_1 = tslib_1.__importDefault(require("typescript"));
const cache_1 = require("./inc/cache");
const getValidPackageName_1 = require("./inc/getValidPackageName");
const logger_1 = require("./logger");
const ModuleResolver_1 = require("./ModuleResolver");
Error.stackTraceLimit = Infinity;
var EngineKind;
(function (EngineKind) {
    EngineKind[EngineKind["Heft"] = 0] = "Heft";
    EngineKind[EngineKind["TTypescript"] = 1] = "TTypescript";
})(EngineKind = exports.EngineKind || (exports.EngineKind = {}));
function noFilter(files) {
    return files;
}
class TypescriptTransformPlugin {
    callee;
    _mutations = [];
    _notifies = [];
    _compilerOptions;
    configurePhase = false;
    constructor() {
        this.callee = new Error('');
        console.log('>>> plugin: %s <<<', this.pluginName);
        if (this.transformToplevelNodes) {
            this.transformToplevelNodes = this.transformToplevelNodes.bind(this);
        }
        this.plugin = this.plugin.bind(this);
    }
    _pluginName;
    get pluginName() {
        if (this._pluginName)
            return this._pluginName;
        const p = (0, common_1.getErrorFrame)(this.callee, 2);
        let pkgpath;
        if (p.includes('(')) {
            pkgpath = (0, path_1.dirname)(p.slice(p.indexOf('(') + 1));
        }
        else {
            pkgpath = (0, path_1.dirname)(p.replace(/^at\s*/, ''));
        }
        if (pkgpath.startsWith('file://')) {
            pkgpath = pkgpath.slice('file://'.length);
        }
        if (!(0, path_1.isAbsolute)(pkgpath)) {
            return pkgpath.replace(/:[0-9:)]+$/, '');
        }
        const pkg = (0, getValidPackageName_1.getValidPackageFile)(pkgpath);
        if (!pkg) {
            return pkgpath.replace(/:[0-9:)]+$/, '');
        }
        this._pluginName = require(pkg).name;
        return this._pluginName;
    }
    get program() {
        this.logger.warn("don't use program if possible");
        return this._program;
    }
    get compilerOptions() {
        if (this._compilerOptions) {
            return this._compilerOptions;
        }
        this.logger.warn('best not to use before');
        return this._program.getCompilerOptions();
    }
    forkProgram(overrideOptions = {}, filter = noFilter) {
        const prevOptions = this.context?.getCompilerOptions() ?? this.program.getCompilerOptions();
        const options = { ...prevOptions, ...overrideOptions };
        const host = typescript_1.default.createCompilerHost(options, true);
        return typescript_1.default.createProgram(filter(this._program.getRootFileNames()), options, host, this._program);
    }
    plugin(program, options) {
        if (this._program === program)
            return this.receiveContext;
        this._program = program;
        this.pluginOptions = options;
        if (options.verbose ||
            process.env.DEBUG_TRANSFORM_COMMON ||
            /\btramsform\b/i.test(process.env.NODE_DEBUG || '')) {
            this.isDebug = true;
        }
        else {
            this.isDebug = false;
        }
        if (!this.logger) {
            if (options.logger) {
                this.logger = (0, logger_1.wrapHeftLogger)(options.logger, this.isDebug);
                this.engine = EngineKind.Heft;
            }
            else {
                this.logger = (0, logger_1.getDebug)(this.isDebug);
                this.engine = EngineKind.TTypescript;
            }
            this.logger.debug('logger started');
        }
        return this.receiveContext;
    }
    _onSubstituteNode(original, mutations, hint, node) {
        // console.log('  ! %s', ts.SyntaxKind[node.kind]);
        for (const { types, method } of mutations) {
            if (types.includes(node.kind)) {
                node = method.call(this, node) || node;
            }
        }
        return original(hint, node);
    }
    _onEmitNode(original, notifies, hint, node, callback) {
        let willEmit = true;
        for (const { types, method } of notifies) {
            if (types.includes(node.kind)) {
                const emit = method.call(this, node);
                if (emit === false) {
                    willEmit = false;
                }
            }
        }
        if (willEmit)
            original(hint, node, callback);
    }
    registerNotifycation(kinds, func) {
        if (!this.configurePhase)
            this.logger.error('registerNotifycation only callable when configure()');
        this._notifies.push({ types: kinds, method: func });
    }
    registerMutation(kinds, func) {
        if (!this.configurePhase)
            this.logger.error('registerMutation only callable when configure()');
        this._mutations.push({ types: kinds, method: func });
    }
    receiveContext(context) {
        this.context = context;
        this._compilerOptions = context.getCompilerOptions();
        this.logger.debug('==== New Context ====', typescript_1.default.ModuleKind[this._compilerOptions.module]);
        this.compilerHost = typescript_1.default.createCompilerHost(this._compilerOptions, true);
        this.compilerHost.getModuleResolutionCache = (0, cache_1.cache)(() => typescript_1.default.createModuleResolutionCache(this.compilerHost.getCurrentDirectory(), this.compilerHost.getCanonicalFileName, this._compilerOptions));
        this.resolver = new ModuleResolver_1.ModuleResolver(this._compilerOptions, this.logger, this.compilerHost);
        if (this.configure) {
            this._notifies.length = 0;
            this._mutations.length = 0;
            this.configurePhase = true;
            this.configure(context, this._compilerOptions);
            this.configurePhase = false;
        }
        if (this._notifies.length) {
            context.onEmitNode = this._onEmitNode.bind(this, context.onEmitNode, this._notifies.slice());
            for (const item of this._notifies) {
                for (const kind of item.types) {
                    context.enableEmitNotification(kind);
                }
            }
        }
        if (this._mutations.length) {
            context.onSubstituteNode = this._onSubstituteNode.bind(this, context.onSubstituteNode, this._mutations.slice());
            for (const item of this._mutations) {
                for (const kind of item.types) {
                    context.enableSubstitution(kind);
                }
            }
        }
        return this.transformer;
    }
    transformer(sourceFile) {
        if (!this.transformToplevelNodes) {
            return sourceFile;
        }
        return typescript_1.default.visitEachChild(sourceFile, this.transformToplevelNodes, this.context);
    }
}
tslib_1.__decorate([
    common_1.bindThis,
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Array, Function]),
    tslib_1.__metadata("design:returntype", void 0)
], TypescriptTransformPlugin.prototype, "registerNotifycation", null);
tslib_1.__decorate([
    common_1.bindThis,
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Array, Function]),
    tslib_1.__metadata("design:returntype", void 0)
], TypescriptTransformPlugin.prototype, "registerMutation", null);
tslib_1.__decorate([
    common_1.bindThis,
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Object]),
    tslib_1.__metadata("design:returntype", Function)
], TypescriptTransformPlugin.prototype, "receiveContext", null);
tslib_1.__decorate([
    common_1.bindThis,
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Object]),
    tslib_1.__metadata("design:returntype", void 0)
], TypescriptTransformPlugin.prototype, "transformer", null);
exports.TypescriptTransformPlugin = TypescriptTransformPlugin;
//# sourceMappingURL=pluginClass.js.map